/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter;

import java.util.Iterator;

import com.ibm.wala.classLoader.IMethod;
import com.ibm.wala.ipa.callgraph.CGNode;
import com.ibm.wala.ipa.callgraph.CallGraph;
import com.ibm.wala.shrikeBT.IInstruction;
import com.ibm.wala.shrikeBT.MethodEditor;

/**
 * 
 * @author Stas Negara
 * 
 */
public class Debugger {

	private static final boolean isInDebugMode = System.getenv("DEBUG_MODE") != null;

	public static void debug(String message) {
		if (isInDebugMode) {
			System.out.println(message);
		}
	}

	public static void debugEntryPoints(CallGraph callGraph) {
		if (isInDebugMode) {
			CGNode fakeRootNode = callGraph.getFakeRootNode();
			System.out.println("FakeRoot CGNode: " + fakeRootNode);
			System.out.println("IR: " + fakeRootNode.getIR());
			Iterator<CGNode> iterator = callGraph.getEntrypointNodes().iterator();
			while (iterator.hasNext()) {
				CGNode node = iterator.next();
				System.out.println("Entrypoint CGNode: " + node);
				System.out.println("IR: " + node.getIR());
			}
		}
	}

	public static void debugEntrypointMethod(IMethod entrypointMethod) {
		if (isInDebugMode) {
			System.out.println("Entrypoint method signature: " + entrypointMethod.getSignature());
		}
	}

	public static void debugTransformedInstructions(MethodEditor methodEditor) {
		if (isInDebugMode) {
			int i = 0;
			System.out.println("Transformed instructions:");
			for (IInstruction instruction : methodEditor.getInstructions()) {
				System.out.println("Instruction" + i + ":" + instruction);
				i++;
			}
		}
	}

	public static void debugSendingNode(CGNode cgNode) {
		if (isInDebugMode) {
			System.out.println("\nSending CGNode: " + cgNode);
			System.out.println("IR: " + cgNode.getIR());
		}
	}

	public static void debugAnalysisTime(long startTime) {
		if (isInDebugMode) {
			System.out.println("Analysis time: " + (System.currentTimeMillis() - startTime));
		}
	}

}
