/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

import com.ibm.wala.classLoader.CallSiteReference;
import com.ibm.wala.ipa.callgraph.CGNode;
import com.ibm.wala.util.io.CommandLine;

import edu.illinois.soter.analysis.OwnershipTransferAnalysis;
import edu.illinois.soter.analysis.actorfoundry.ActorFoundryAnalysis;
import edu.illinois.soter.analysis.scala.ScalaAnalysis;
import edu.illinois.soter.messagedata.MessageInvocation;
import edu.illinois.soter.transformation.CodeTransformer;

/**
 * 
 * @author Stas Negara
 * 
 */
public class Soter {

	private static final char portableFilePathSeparator = '/';

	private static final String ACTOR_FOUNDRY_PROGRAM = "ActorFoundry";
	private static final String SCALA_PROGRAM = "Scala";

	public static void main(String[] args) {
		long startTime = System.currentTimeMillis();
		Properties analysisProperties = CommandLine.parse(args);
		OwnershipTransferAnalysis actorAnalysis = createOwnershipTransferAnalysis(analysisProperties);
		try {
			Map<CGNode, Map<CallSiteReference, MessageInvocation>> markedMessageInvocations = actorAnalysis.perform();
			if (Boolean.valueOf(analysisProperties.getProperty("showAnalysisResults"))) {
				showAnalysisResults(markedMessageInvocations);
			}
			if (actorAnalysis.canTransform() && Boolean.valueOf(analysisProperties.getProperty("transform"))) {
				String outputPath = analysisProperties.getProperty("output").replace('\\', portableFilePathSeparator) + portableFilePathSeparator;
				new CodeTransformer().perform(outputPath, markedMessageInvocations);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		Debugger.debugAnalysisTime(startTime);
	}

	private static OwnershipTransferAnalysis createOwnershipTransferAnalysis(Properties analysisProperties) {
		String classPath = analysisProperties.getProperty("classpath");
		String analyzedProgramKind = analysisProperties.getProperty("analyzedProgramKind");
		OwnershipTransferAnalysis actorAnalysis = null;
		if (analyzedProgramKind.equalsIgnoreCase(ACTOR_FOUNDRY_PROGRAM)) {
			actorAnalysis = new ActorFoundryAnalysis(classPath);
		} else if (analyzedProgramKind.equalsIgnoreCase(SCALA_PROGRAM)) {
			actorAnalysis = new ScalaAnalysis(classPath, analysisProperties.getProperty("entryClassName"));
		} else {
			throw new RuntimeException("Unsupported program kind: \"" + analyzedProgramKind + "\"");
		}
		return actorAnalysis;
	}

	private static void showAnalysisResults(Map<CGNode, Map<CallSiteReference, MessageInvocation>> messageInvocations) {
		for (Entry<CGNode, Map<CallSiteReference, MessageInvocation>> mapEntry : messageInvocations.entrySet()) {
			for (MessageInvocation messageInvocation : mapEntry.getValue().values()) {
				System.out.println(messageInvocation);
			}
		}
	}

}
