/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.analysis.scala;

import java.util.Map;
import java.util.Set;

import com.ibm.wala.classLoader.IClass;
import com.ibm.wala.ipa.callgraph.MethodTargetSelector;
import com.ibm.wala.ipa.cha.IClassHierarchy;
import com.ibm.wala.ipa.summaries.BypassMethodTargetSelector;
import com.ibm.wala.ipa.summaries.MethodSummary;
import com.ibm.wala.types.MemberReference;
import com.ibm.wala.types.MethodReference;
import com.ibm.wala.util.strings.Atom;

/**
 * 
 * @author Stas Negara
 * 
 */
public class ScalaBypassMethodTargetSelector extends BypassMethodTargetSelector {

	private final IClass actorClass;

	public ScalaBypassMethodTargetSelector(IClass actorClass, MethodTargetSelector parent, Map<MethodReference, MethodSummary> methodSummaries, Set<Atom> ignoredPackages,
			IClassHierarchy classHierarchy) {
		super(parent, methodSummaries, ignoredPackages, classHierarchy);
		this.actorClass = actorClass;
	}

	@Override
	protected boolean canIgnore(MemberReference methodReference) {
		String methodSignature = methodReference.getSignature();
		//Block interactions with Scala Scheduler to ensure modularity of the analysis
		if (methodSignature.endsWith(ScalaAnalysis.BANG_SIGNATURE_SUFFIX) || methodSignature.endsWith(ScalaAnalysis.BANG_QMARK_SIGNATURE_SUFFIX)
				|| methodSignature.endsWith(ScalaAnalysis.START_SIGNATURE_SUFFIX) || methodSignature.endsWith(ScalaAnalysis.EXIT_SIGNATURE_SUFFIX)) {
			IClass declaringClass = cha.lookupClass(methodReference.getDeclaringClass());
			if (cha.implementsInterface(declaringClass, actorClass)) {
				return true;
			}
		}
		return super.canIgnore(methodReference);
	}

}
