/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.analysis.transferfunctionproviders;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ibm.wala.classLoader.CallSiteReference;
import com.ibm.wala.dataflow.graph.AbstractMeetOperator;
import com.ibm.wala.dataflow.graph.BitVectorIdentity;
import com.ibm.wala.dataflow.graph.BitVectorUnion;
import com.ibm.wala.dataflow.graph.BitVectorUnionVector;
import com.ibm.wala.dataflow.graph.ITransferFunctionProvider;
import com.ibm.wala.fixedpoint.impl.UnaryOperator;
import com.ibm.wala.fixpoint.BitVectorVariable;
import com.ibm.wala.ipa.callgraph.CGNode;
import com.ibm.wala.ipa.callgraph.CallGraph;
import com.ibm.wala.util.intset.BitVector;

/**
 * 
 * @author Stas Negara
 * 
 */
public class GlobalLiveVariablesTransferFunctionProvider implements ITransferFunctionProvider<CGNode, BitVectorVariable> {

	private CallGraph callGraph;
	private Set<CGNode> relevantCGNodes;
	private Map<CGNode, Map<CallSiteReference, BitVector>> interProceduralLiveVariables;

	public GlobalLiveVariablesTransferFunctionProvider(CallGraph callGraph, Set<CGNode> relevantCGNodes, Map<CGNode, Map<CallSiteReference, BitVector>> interProceduralLiveVariables) {
		this.callGraph = callGraph;
		this.relevantCGNodes = relevantCGNodes;
		this.interProceduralLiveVariables = interProceduralLiveVariables;
	}

	public AbstractMeetOperator<BitVectorVariable> getMeetOperator() {
		return BitVectorUnion.instance();
	}

	public boolean hasEdgeTransferFunctions() {
		return true;
	}

	public UnaryOperator<BitVectorVariable> getEdgeTransferFunction(CGNode src, CGNode dst) {
		if (relevantCGNodes.contains(src) && relevantCGNodes.contains(dst)) {
			Map<CallSiteReference, BitVector> srcNodeLiveVariabes = interProceduralLiveVariables.get(src);
			Iterator<CallSiteReference> callSitesIterator = callGraph.getPossibleSites(src, dst);
			BitVector resultBitVector = new BitVector();
			while (callSitesIterator.hasNext()) {
				CallSiteReference callSiteReference = callSitesIterator.next();
				BitVector callSiteBitVector = srcNodeLiveVariabes.get(callSiteReference);
				if (callSiteBitVector == null) {
					throw new RuntimeException("Could not find bit vector for call site: " + callSiteReference);
				}
				resultBitVector.or(callSiteBitVector);
			}
			return new BitVectorUnionVector(resultBitVector);
		}
		return BitVectorIdentity.instance();//TODO: Will it improve performance if values are filtered out instead of propagated?
	}

	public boolean hasNodeTransferFunctions() {
		return false;
	}

	public UnaryOperator<BitVectorVariable> getNodeTransferFunction(CGNode node) {
		return null;
	}

}
