/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.analysis.transferfunctionproviders;

import java.util.Iterator;
import java.util.LinkedList;

import com.ibm.wala.dataflow.graph.AbstractMeetOperator;
import com.ibm.wala.dataflow.graph.BitVectorKillGen;
import com.ibm.wala.dataflow.graph.BitVectorUnion;
import com.ibm.wala.dataflow.graph.ITransferFunctionProvider;
import com.ibm.wala.fixedpoint.impl.UnaryOperator;
import com.ibm.wala.fixpoint.BitVectorVariable;
import com.ibm.wala.ipa.callgraph.CGNode;
import com.ibm.wala.ipa.callgraph.propagation.HeapModel;
import com.ibm.wala.ipa.callgraph.propagation.PointerKey;
import com.ibm.wala.ssa.ISSABasicBlock;
import com.ibm.wala.ssa.SSAInstruction;
import com.ibm.wala.util.intset.BitVector;
import com.ibm.wala.util.intset.OrdinalSetMapping;

/**
 * 
 * @author Stas Negara
 * 
 */
public class LocalLiveVariablesTransferFunctionProvider implements ITransferFunctionProvider<ISSABasicBlock, BitVectorVariable> {

	private HeapModel heapModel;
	private CGNode cgNode;
	private OrdinalSetMapping<PointerKey> values;

	public LocalLiveVariablesTransferFunctionProvider(HeapModel heapModel, CGNode cgNode, OrdinalSetMapping<PointerKey> values) {
		this.heapModel = heapModel;
		this.cgNode = cgNode;
		this.values = values;
	}

	public AbstractMeetOperator<BitVectorVariable> getMeetOperator() {
		return BitVectorUnion.instance();
	}

	public boolean hasNodeTransferFunctions() {
		return true;
	}

	public UnaryOperator<BitVectorVariable> getNodeTransferFunction(ISSABasicBlock node) {
		BitVector gen = new BitVector();
		BitVector kill = new BitVector();
		Iterator<SSAInstruction> instructionsIterator = node.iterator();
		//Process instructions backwards (in a reverse order) to prepare the kill and gen bitvectors for the whole basic block. 
		LinkedList<SSAInstruction> reverseInstructions = new LinkedList<SSAInstruction>();
		while (instructionsIterator.hasNext()) {
			reverseInstructions.addFirst(instructionsIterator.next());
		}
		for (SSAInstruction instruction : reverseInstructions) {
			for (int i = 0; i < instruction.getNumberOfDefs(); i++) {
				int bitIndex = getBitIndexForVariable(instruction.getDef(i));
				kill.set(bitIndex);
				gen.clear(bitIndex);
			}
			for (int i = 0; i < instruction.getNumberOfUses(); i++) {
				int use = instruction.getUse(i);
				if (use != -1) { //In PHI instructions some uses may be -1 
					int bitIndex = getBitIndexForVariable(use);
					gen.set(bitIndex);
					kill.clear(bitIndex); //Actually, this is redundant. For SSA form in a basic block uses can not precede defs.
				}
			}
		}
		return new BitVectorKillGen(kill, gen);
	}

	private int getBitIndexForVariable(int var) {
		PointerKey pointerKey = heapModel.getPointerKeyForLocal(cgNode, var);
		return values.add(pointerKey);
	}

	public boolean hasEdgeTransferFunctions() {
		return false;
	}

	public UnaryOperator<BitVectorVariable> getEdgeTransferFunction(ISSABasicBlock src, ISSABasicBlock dst) {
		return null;
	}

}
