/**
 * Developed by: The Open Systems Lab
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             Urbana, IL 61801
 *             http://osl.cs.uiuc.edu
 *
 * Contact: http://osl.cs.uiuc.edu/soter
 *
 * Copyright (c) 1998-2011
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * 
 * Distributed under license: http://osl.cs.uiuc.edu/soter/LICENSE
 * 
 */
package edu.illinois.soter.messagedata;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import com.ibm.wala.classLoader.IMethod;
import com.ibm.wala.ipa.callgraph.propagation.AllocationSiteInNode;
import com.ibm.wala.ipa.callgraph.propagation.ConstantKey;

import edu.illinois.soter.analysis.AnalysisHelper;
import edu.illinois.soter.analysis.OwnershipTransferAnalysis;

/**
 * 
 * @author Stas Negara
 * 
 */
public class MessageArgument {
	final int transformationBytecodeIndex;
	final Map<Object, Boolean> argumentItems = new HashMap<Object, Boolean>();
	boolean passByValue = false;

	public MessageArgument(int transformationBytecodeIndex, Set<Object> argumentItems) {
		this.transformationBytecodeIndex = transformationBytecodeIndex;
		for (Object item : argumentItems) {
			if (!OwnershipTransferAnalysis.IS_IGNORING_STATIC_FIELDS || !isPointedByJDKStaticField(item)) {
				this.argumentItems.put(item, Boolean.FALSE);
			}
		}
	}

	private boolean isPointedByJDKStaticField(Object item) {
		if (item instanceof AllocationSiteInNode) {
			AllocationSiteInNode allocationSite = (AllocationSiteInNode) item;
			IMethod allocationMethod = allocationSite.getNode().getMethod();
			if (AnalysisHelper.isClassInitMethod(allocationMethod) && AnalysisHelper.isJDKClass(allocationMethod.getDeclaringClass())) {
				return true;
			}
		}
		return false;
	}

	public boolean isPassByValue() {
		return passByValue;
	}

	public int getTransformationBytecodeIndex() {
		return transformationBytecodeIndex;
	}

	/**
	 * Checks if "this" MessageArgument and the provided "argument"
	 * MessageArgument may have common items, in which case they both should be
	 * passed by value.
	 * 
	 * @param argument
	 */
	public void check(MessageArgument argument) {
		for (Object argumentItem : argumentItems.keySet()) {
			if (argument.argumentItems.containsKey(argumentItem)) {
				passByValue = true;
				argument.passByValue = true;
				argumentItems.put(argumentItem, Boolean.TRUE);
				argument.argumentItems.put(argumentItem, Boolean.TRUE);
			}
		}
	}

	/**
	 * Checks if MessageArgument and the provided set of live items may have
	 * common items, in which case the MessageArgument should be passed by
	 * value.
	 * 
	 * @param liveItems
	 */
	public void check(Set<Object> liveItems) {
		for (Object argumentItem : argumentItems.keySet()) {
			if (!(argumentItem instanceof ConstantKey) && liveItems.contains(argumentItem)) {
				passByValue = true;
				argumentItems.put(argumentItem, Boolean.TRUE);
			}
		}
	}

	public String toString() {
		StringBuffer returnValue = new StringBuffer();
		returnValue.append("Transformation bytecode index: ").append(transformationBytecodeIndex).append("\n");
		for (Entry<Object, Boolean> entry : argumentItems.entrySet()) {
			returnValue.append("ArgumentItem: ").append(entry.getValue().booleanValue() ? "*" : "").append(entry.getKey()).append("\n");
		}
		returnValue.append("Pass by value: ").append(passByValue).append("\n");
		return returnValue.toString();
	}

}
